@@ -32,7 +32,8 @@ impl Plugin for OverlayPlugin {
             .add_plugin(ExtractComponentPlugin::<Handle<OverlayMaterial>>::default())
             .add_plugin(RenderAssetPlugin::<OverlayMaterial>::default())
             .init_resource::<ScreenOverlay>()
-            .add_startup_system(setup);
+            .add_startup_system(setup)
+            .add_system_to_stage(CoreStage::PostUpdate, update);
 
         if let Ok(render_app) = app.get_sub_app_mut(RenderApp) {
             render_app
@@ -48,105 +49,15 @@ impl Plugin for OverlayPlugin {
     }
 }
 
-fn setup(
-    mut commands: Commands,
-    overlay: Res<ScreenOverlay>,
-    mut meshes: ResMut<Assets<Mesh>>,
-    mut materials: ResMut<Assets<OverlayMaterial>>,
-) {
-    let irradiance_resolve = overlay.irradiance_resolve.clone();
-    let albedo_resolve = overlay.albedo_resolve.clone();
-
-    commands.spawn_bundle(MaterialMeshBundle {
-        mesh: meshes.add(shape::Quad::new(Vec2::ZERO).into()),
-        material: materials.add(OverlayMaterial {
-            irradiance_image: irradiance_resolve,
-            albedo_image: albedo_resolve,
-        }),
-        ..Default::default()
-    });
-}
-
-fn extract_screen_overlay(mut commands: Commands, screen_overlay: Res<ScreenOverlay>) {
-    commands.insert_resource(screen_overlay.clone());
-}
-
-fn prepare_screen_overlay(
-    mut commands: Commands,
-    images: Res<RenderAssets<Image>>,
-    overlay: Res<ScreenOverlay>,
-    msaa: Res<Msaa>,
-    render_device: Res<RenderDevice>,
-    mut texture_cache: ResMut<TextureCache>,
-) {
-    let irradiance_depth = texture_cache.get(
-        &render_device,
-        TextureDescriptor {
-            label: Some("volume_overlay_depth_texture"),
-            size: overlay.irradiance_size,
-            mip_level_count: 1,
-            sample_count: msaa.samples,
-            dimension: TextureDimension::D2,
-            format: TextureFormat::Depth32Float,
-            usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
-        },
-    );
-
-    let albedo_depth = texture_cache.get(
-        &render_device,
-        TextureDescriptor {
-            label: Some("volume_overlay_depth_texture"),
-            size: overlay.albedo_size,
-            mip_level_count: 1,
-            sample_count: msaa.samples,
-            dimension: TextureDimension::D2,
-            format: TextureFormat::Depth32Float,
-            usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
-        },
-    );
-
-    let retrieve_textures = || {
-        let irradiance = images.get(&overlay.irradiance)?.texture_view.clone();
-        let irradiance_resolve = images
-            .get(&overlay.irradiance_resolve)?
-            .texture_view
-            .clone();
-        let albedo = images.get(&overlay.albedo)?.texture_view.clone();
-        let albedo_resolve = images.get(&overlay.albedo_resolve)?.texture_view.clone();
-        Some((irradiance, irradiance_resolve, albedo, albedo_resolve))
-    };
-
-    if let Some((irradiance, irradiance_resolve, albedo, albedo_resolve)) = retrieve_textures() {
-        commands.insert_resource(GpuScreenOverlay {
-            irradiance,
-            irradiance_resolve,
-            irradiance_depth,
-            albedo,
-            albedo_resolve,
-            albedo_depth,
-        });
-    }
-}
-
 #[derive(Debug, Clone)]
 pub struct ScreenOverlay {
     pub irradiance_size: Extent3d,
+    pub irradiance_sampled: Option<Handle<Image>>,
     pub irradiance: Handle<Image>,
-    pub irradiance_resolve: Handle<Image>,
 
     pub albedo_size: Extent3d,
+    pub albedo_sampled: Option<Handle<Image>>,
     pub albedo: Handle<Image>,
-    pub albedo_resolve: Handle<Image>,
-}
-
-pub struct GpuScreenOverlay {
-    pub irradiance: TextureView,
-    pub irradiance_resolve: TextureView,
-    pub irradiance_depth: CachedTexture,
-
-    pub albedo: TextureView,
-    pub albedo_resolve: TextureView,
-    pub albedo_depth: CachedTexture,
 }
 
 impl FromWorld for ScreenOverlay {
@@ -175,12 +86,18 @@ impl FromWorld for ScreenOverlay {
         image.texture_descriptor.usage = TextureUsages::COPY_DST
             | TextureUsages::RENDER_ATTACHMENT
             | TextureUsages::TEXTURE_BINDING;
+        image.sampler_descriptor.mag_filter = FilterMode::Linear;
+        image.sampler_descriptor.min_filter = FilterMode::Linear;
 
         image.texture_descriptor.sample_count = samples;
-        let irradiance = images.add(image.clone());
+        let irradiance_sampled = if samples > 1 {
+            Some(images.add(image.clone()))
+        } else {
+            None
+        };
 
         image.texture_descriptor.sample_count = 1;
-        let irradiance_resolve = images.add(image);
+        let irradiance = images.add(image);
 
         let albedo_size = Extent3d {
             width,
@@ -196,22 +113,74 @@ impl FromWorld for ScreenOverlay {
         image.texture_descriptor.usage = TextureUsages::COPY_DST
             | TextureUsages::RENDER_ATTACHMENT
             | TextureUsages::TEXTURE_BINDING;
-        image.sampler_descriptor.mag_filter = FilterMode::Linear;
-        image.sampler_descriptor.min_filter = FilterMode::Linear;
 
         image.texture_descriptor.sample_count = samples;
-        let albedo = images.add(image.clone());
+        let albedo_sampled = if samples > 1 {
+            Some(images.add(image.clone()))
+        } else {
+            None
+        };
 
         image.texture_descriptor.sample_count = 1;
-        let albedo_resolve = images.add(image);
+        let albedo = images.add(image);
 
         Self {
             irradiance_size,
+            irradiance_sampled,
             irradiance,
-            irradiance_resolve,
             albedo_size,
+            albedo_sampled,
             albedo,
-            albedo_resolve,
+        }
+    }
+}
+
+pub struct GpuScreenOverlay {
+    pub irradiance: TextureView,
+    pub irradiance_sampled: Option<TextureView>,
+    pub irradiance_depth: CachedTexture,
+
+    pub albedo: TextureView,
+    pub albedo_sampled: Option<TextureView>,
+    pub albedo_depth: CachedTexture,
+}
+
+impl GpuScreenOverlay {
+    pub fn irradiance_color_attachment(
+        &self,
+        ops: Operations<wgpu::Color>,
+    ) -> RenderPassColorAttachment {
+        RenderPassColorAttachment {
+            view: if let Some(sampled) = &self.irradiance_sampled {
+                sampled
+            } else {
+                &self.irradiance
+            },
+            resolve_target: if self.irradiance_sampled.is_some() {
+                Some(&self.irradiance)
+            } else {
+                None
+            },
+            ops,
+        }
+    }
+
+    pub fn albedo_color_attachment(
+        &self,
+        ops: Operations<wgpu::Color>,
+    ) -> RenderPassColorAttachment {
+        RenderPassColorAttachment {
+            view: if let Some(sampled) = &self.albedo_sampled {
+                sampled
+            } else {
+                &self.albedo
+            },
+            resolve_target: if self.albedo_sampled.is_some() {
+                Some(&self.albedo)
+            } else {
+                None
+            },
+            ops,
         }
     }
 }
@@ -219,8 +188,8 @@ impl FromWorld for ScreenOverlay {
 #[derive(Debug, Clone, TypeUuid)]
 #[uuid = "3eb25222-95fd-11ec-b909-0242ac120002"]
 pub struct OverlayMaterial {
-    pub irradiance_image: Handle<Image>,
-    pub albedo_image: Handle<Image>,
+    pub irradiance: Handle<Image>,
+    pub albedo: Handle<Image>,
 }
 
 #[derive(Clone)]
@@ -245,16 +214,13 @@ impl RenderAsset for OverlayMaterial {
         material: Self::ExtractedAsset,
         (render_device, material_pipeline, images): &mut SystemParamItem<Self::Param>,
     ) -> Result<Self::PreparedAsset, PrepareAssetError<Self::ExtractedAsset>> {
-        let irradiance = if let Some(result) = images.get(&material.irradiance_image) {
-            result
-        } else {
-            return Err(PrepareAssetError::RetryNextUpdate(material));
+        let irradiance = match images.get(&material.irradiance) {
+            Some(image) => image,
+            None => return Err(PrepareAssetError::RetryNextUpdate(material)),
         };
-
-        let albedo = if let Some(result) = images.get(&material.albedo_image) {
-            result
-        } else {
-            return Err(PrepareAssetError::RetryNextUpdate(material));
+        let albedo = match images.get(&material.albedo) {
+            Some(image) => image,
+            None => return Err(PrepareAssetError::RetryNextUpdate(material)),
         };
 
         let bind_group = render_device.create_bind_group(&BindGroupDescriptor {
@@ -308,7 +274,7 @@ impl SpecializedMaterial for OverlayMaterial {
                     binding: 0,
                     visibility: ShaderStages::FRAGMENT,
                     ty: BindingType::Texture {
-                        sample_type: TextureSampleType::Float { filterable: true },
+                        sample_type: TextureSampleType::default(),
                         view_dimension: TextureViewDimension::D2,
                         multisampled: false,
                     },
@@ -324,7 +290,7 @@ impl SpecializedMaterial for OverlayMaterial {
                     binding: 2,
                     visibility: ShaderStages::FRAGMENT,
                     ty: BindingType::Texture {
-                        sample_type: TextureSampleType::Float { filterable: true },
+                        sample_type: TextureSampleType::default(),
                         view_dimension: TextureViewDimension::D2,
                         multisampled: false,
                     },
@@ -361,6 +327,108 @@ impl SpecializedMaterial for OverlayMaterial {
     }
 }
 
+fn setup(
+    mut commands: Commands,
+    overlay: Res<ScreenOverlay>,
+    mut meshes: ResMut<Assets<Mesh>>,
+    mut materials: ResMut<Assets<OverlayMaterial>>,
+) {
+    let irradiance = overlay.irradiance.clone();
+    let albedo = overlay.albedo.clone();
+
+    commands.spawn_bundle(MaterialMeshBundle {
+        mesh: meshes.add(shape::Quad::new(Vec2::ZERO).into()),
+        material: materials.add(OverlayMaterial { irradiance, albedo }),
+        ..Default::default()
+    });
+}
+
+fn update(mut images: ResMut<Assets<Image>>, mut overlay: ResMut<ScreenOverlay>, msaa: Res<Msaa>) {
+    if msaa.is_changed() {
+        overlay.irradiance_sampled = if msaa.samples > 1 {
+            let mut image = images.get(overlay.irradiance.clone()).unwrap().clone();
+            image.texture_descriptor.sample_count = msaa.samples;
+            Some(images.add(image))
+        } else {
+            None
+        };
+
+        overlay.albedo_sampled = if msaa.samples > 1 {
+            let mut image = images.get(overlay.albedo.clone()).unwrap().clone();
+            image.texture_descriptor.sample_count = msaa.samples;
+            Some(images.add(image))
+        } else {
+            None
+        };
+    }
+}
+
+fn extract_screen_overlay(mut commands: Commands, screen_overlay: Res<ScreenOverlay>) {
+    commands.insert_resource(screen_overlay.clone());
+}
+
+fn prepare_screen_overlay(
+    mut commands: Commands,
+    msaa: Res<Msaa>,
+    images: Res<RenderAssets<Image>>,
+    overlay: Res<ScreenOverlay>,
+    render_device: Res<RenderDevice>,
+    mut texture_cache: ResMut<TextureCache>,
+) {
+    let irradiance_depth = texture_cache.get(
+        &render_device,
+        TextureDescriptor {
+            label: Some("volume_overlay_depth_texture"),
+            size: overlay.irradiance_size,
+            mip_level_count: 1,
+            sample_count: msaa.samples,
+            dimension: TextureDimension::D2,
+            format: TextureFormat::Depth32Float,
+            usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
+        },
+    );
+
+    let albedo_depth = texture_cache.get(
+        &render_device,
+        TextureDescriptor {
+            label: Some("volume_overlay_depth_texture"),
+            size: overlay.albedo_size,
+            mip_level_count: 1,
+            sample_count: msaa.samples,
+            dimension: TextureDimension::D2,
+            format: TextureFormat::Depth32Float,
+            usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
+        },
+    );
+
+    let retrieve_textures = || {
+        let irradiance_sampled = match &overlay.irradiance_sampled {
+            Some(handle) => Some(images.get(handle)?.texture_view.clone()),
+            None => None,
+        };
+        let irradiance = images.get(&overlay.irradiance)?.texture_view.clone();
+
+        let albedo_sampled = match &overlay.albedo_sampled {
+            Some(handle) => Some(images.get(handle)?.texture_view.clone()),
+            None => None,
+        };
+        let albedo = images.get(&overlay.albedo)?.texture_view.clone();
+
+        Some((irradiance_sampled, irradiance, albedo_sampled, albedo))
+    };
+
+    if let Some((irradiance_sampled, irradiance, albedo_sampled, albedo)) = retrieve_textures() {
+        commands.insert_resource(GpuScreenOverlay {
+            irradiance_sampled,
+            irradiance,
+            irradiance_depth,
+            albedo_sampled,
+            albedo,
+            albedo_depth,
+        });
+    }
+}
+
 fn prepare_overlay_phase(mut commands: Commands, views: Query<Entity, With<Volume>>) {
     for view in views.iter() {
         commands
