@@ -89,16 +89,14 @@ pub struct TracingPipeline {
 impl FromWorld for TracingPipeline {
     fn from_world(world: &mut World) -> Self {
         let mesh_pipeline = world.get_resource::<MeshPipeline>().unwrap().clone();
-
         let render_device = world.get_resource::<RenderDevice>().unwrap();
-
         let material_layout = StandardMaterial::bind_group_layout(render_device);
 
         let anisotropic_layout_entries = (0..6).map(|direction| BindGroupLayoutEntry {
             binding: direction,
             visibility: ShaderStages::FRAGMENT,
             ty: BindingType::Texture {
-                sample_type: TextureSampleType::Float { filterable: true },
+                sample_type: TextureSampleType::default(),
                 view_dimension: TextureViewDimension::D3,
                 multisampled: false,
             },
@@ -108,6 +106,22 @@ impl FromWorld for TracingPipeline {
             BindGroupLayoutEntry {
                 binding: 6,
                 visibility: ShaderStages::FRAGMENT,
+                ty: BindingType::Texture {
+                    sample_type: TextureSampleType::default(),
+                    view_dimension: TextureViewDimension::D3,
+                    multisampled: false,
+                },
+                count: None,
+            },
+            BindGroupLayoutEntry {
+                binding: 7,
+                visibility: ShaderStages::FRAGMENT,
+                ty: BindingType::Sampler(SamplerBindingType::Filtering),
+                count: None,
+            },
+            BindGroupLayoutEntry {
+                binding: 8,
+                visibility: ShaderStages::FRAGMENT,
                 ty: BindingType::Buffer {
                     ty: BufferBindingType::Uniform,
                     has_dynamic_offset: false,
@@ -116,7 +130,7 @@ impl FromWorld for TracingPipeline {
                 count: None,
             },
             BindGroupLayoutEntry {
-                binding: 7,
+                binding: 9,
                 visibility: ShaderStages::FRAGMENT,
                 ty: BindingType::Buffer {
                     ty: BufferBindingType::Uniform,
@@ -125,22 +139,6 @@ impl FromWorld for TracingPipeline {
                 },
                 count: None,
             },
-            BindGroupLayoutEntry {
-                binding: 8,
-                visibility: ShaderStages::FRAGMENT,
-                ty: BindingType::Texture {
-                    sample_type: TextureSampleType::Float { filterable: true },
-                    view_dimension: TextureViewDimension::D3,
-                    multisampled: false,
-                },
-                count: None,
-            },
-            BindGroupLayoutEntry {
-                binding: 9,
-                visibility: ShaderStages::FRAGMENT,
-                ty: BindingType::Sampler(SamplerBindingType::Filtering),
-                count: None,
-            },
         ];
 
         let tracing_layout_entries = anisotropic_layout_entries
@@ -412,23 +410,23 @@ pub fn queue_tracing_bind_groups(
         let tracing_bindings = vec![
             BindGroupEntry {
                 binding: 6,
+                resource: BindingResource::TextureView(&volume_bindings.voxel_texture.default_view),
+            },
+            BindGroupEntry {
+                binding: 7,
+                resource: BindingResource::Sampler(&volume_bindings.texture_sampler),
+            },
+            BindGroupEntry {
+                binding: 8,
                 resource: BindingResource::Buffer(BufferBinding {
                     buffer: &directions.0,
                     offset: 0,
                     size: None,
                 }),
             },
-            BindGroupEntry {
-                binding: 7,
-                resource: volume_meta.volume_uniforms.binding().unwrap(),
-            },
-            BindGroupEntry {
-                binding: 8,
-                resource: BindingResource::TextureView(&volume_bindings.voxel_texture.default_view),
-            },
             BindGroupEntry {
                 binding: 9,
-                resource: BindingResource::Sampler(&volume_bindings.texture_sampler),
+                resource: volume_meta.volume_uniforms.binding().unwrap(),
             },
         ];
 
@@ -592,14 +590,10 @@ impl render_graph::Node for TracingPassNode {
         {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_opaque_pass"),
-                color_attachments: &[RenderPassColorAttachment {
-                    view: &overlay.irradiance,
-                    resolve_target: Some(&overlay.irradiance_resolve),
-                    ops: Operations {
-                        load: LoadOp::Clear(Color::NONE.into()),
-                        store: true,
-                    },
-                }],
+                color_attachments: &[overlay.irradiance_color_attachment(Operations {
+                    load: LoadOp::Clear(Color::NONE.into()),
+                    store: true,
+                })],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
                     view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
@@ -629,14 +623,10 @@ impl render_graph::Node for TracingPassNode {
         {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_alpha_mask_pass"),
-                color_attachments: &[RenderPassColorAttachment {
-                    view: &overlay.irradiance,
-                    resolve_target: Some(&overlay.irradiance_resolve),
-                    ops: Operations {
-                        load: LoadOp::Load,
-                        store: true,
-                    },
-                }],
+                color_attachments: &[overlay.irradiance_color_attachment(Operations {
+                    load: LoadOp::Load,
+                    store: true,
+                })],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
                     view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
@@ -666,14 +656,10 @@ impl render_graph::Node for TracingPassNode {
         {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_transparent_pass"),
-                color_attachments: &[RenderPassColorAttachment {
-                    view: &overlay.irradiance,
-                    resolve_target: Some(&overlay.irradiance_resolve),
-                    ops: Operations {
-                        load: LoadOp::Load,
-                        store: true,
-                    },
-                }],
+                color_attachments: &[overlay.irradiance_color_attachment(Operations {
+                    load: LoadOp::Load,
+                    store: true,
+                })],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
                     view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
@@ -703,14 +689,10 @@ impl render_graph::Node for TracingPassNode {
         {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_ambient_occlusion_pass"),
-                color_attachments: &[RenderPassColorAttachment {
-                    view: &overlay.irradiance,
-                    resolve_target: Some(&overlay.irradiance_resolve),
-                    ops: Operations {
-                        load: LoadOp::Load,
-                        store: true,
-                    },
-                }],
+                color_attachments: &[overlay.irradiance_color_attachment(Operations {
+                    load: LoadOp::Load,
+                    store: true,
+                })],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
                     view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
